<?php

/*
 * ==========================================================
 * EXCHANGE_CODE.PHP
 * ==========================================================
 *
 * © 2022-2023 boxcoin.dev. All rights reserved.
 *
 */

function bxc_html_cryptocurrencies_list($all = true) {
    $code = '<div class="bxc-select-ul">';
    $items = bxc_get_cryptocurrency_codes($all ? false : 'eth');
    $allowed = false;
    if ($all) {
        if (bxc_settings_get('ln-node-active')) {
            array_push($items['BTC'], 'btc_ln');
        }
    } else {
        $items = ['BTC' => ['btc'], 'ETH' => $items];
        $allowed = str_replace(' ', '', bxc_settings_get('exchange-receive-list'));
        if ($allowed) {
            $allowed = explode(',', $allowed);
        }
    }
    foreach ($items as $value) {
        for ($i = 0; $i < count($value); $i++) {
            $item = $value[$i];
            if ((!$allowed || in_array($item, $allowed)) && ($item == 'btc_ln' || bxc_settings_get_address('address-' . $item) || bxc_settings_get_address($item))) {
                $network = bxc_crypto_get_network($item);
                $name = bxc_crypto_name($item, true);
                $code .= '<div data-value="' . $item . '" data-crypto="true" data-network="' . $network . '" data-name="' . $name . '" class="bxc-flex"><img src="' . bxc_crypto_get_image($item) . '" alt="' . $item . '"><div><span>' . $name . '</span><span>' . strtoupper(bxc_crypto_get_base_code($item)) . ' • ' . ($network ? bxc_crypto_name($network, true) : $name) . '</span></div></div>';
            }
        }
    }
    echo $code . '</div>';
}

function bxc_html_currencies_list($all = false) {
    $code = '<div class="bxc-select-ul">';
    $items = json_decode(file_get_contents(__DIR__ . '/../../resources/currencies' . ($all ? '' : '_' . (bxc_settings_get('stripe-active') ? 'stripe' : 'verifone')) . '.json'), true);
    for ($i = 0; $i < count($items); $i++) {
        $item = $items[$i];
        $code .= '<div data-value="' . strtolower($item[0]) . '" data-currency="true" class="bxc-flex"><img src="' . BXC_URL . 'media/flags/' . strtolower(substr($item[0], 0, 2)) . '.svg" alt="' . $item[0] . '"><div><span>' . $item[1] . '</span><span>' . $item[0] . '</span></div></div>';
    }
    echo $code . '</div>';
}

function bxc_exchange_init() {
    $default_currency = strtoupper(bxc_settings_get('currency', 'USD'));
    $default_cryptocurrency = bxc_settings_get('exchange-default-get', 'btc');
    $identity_type = bxc_settings_get('exchange-identity-type'); 
    $colors = [bxc_settings_get('color-1'), bxc_settings_get('color-2'), bxc_settings_get('color-3')];
    $css = '';
    $css_cnt = '';
    if (bxc_settings_get('exchange-currency-switch')) {
        $default_currency = bxc_isset(bxc_ip_info('currency'), 'currency', $default_currency);
    }
    if ($colors[0]) {
        $css .= '.bxc-btn { background-color: ' . $colors[0] . '; }';
        $css .= '.bxc-input input:focus { border-color: ' . $colors[0] . '; box-shadow: 0 0 5px ' . $colors[0] . '; }';
        $css .= '.bxc-title > i:hover { color: ' . $colors[0] . '; }';
    }
    if ($colors[1]) {
        $css .= '.bxc-btn:hover, .bxc-btn:active { background-color: ' . $colors[1] . '; }';
        $css .= '.bxc-title { color: ' . $colors[1] . '; }';
    }
    if ($colors[2]) {
        $css .= '.bxc-box, .bxc-text, .bxc-footer, .bxc-footer a, .bxc-input input, .bxc-input select, .bxc-input textarea { color: ' . $colors[2] . '; }';
    }
    if ($css) {
        echo '<style>' . $css . '</style>';
    }
    if (bxc_is_rtl(bxc_language())) {
        $css_cnt .= ' bxc-rtl';
    }
    ?>
    <div id="boxcoin-exchange" class="bxc-box<?php echo $css_cnt ?>">
        <div id="bxc-panel-start" class="bxc-panel bxc-active">
            <div class="bxc-title">
                <?php bxc_e(bxc_settings_get('exchange-texts-you-send', 'You send')) ?><i id="bxc-invert" class="bxc-icon-shuffle"></i>
            </div>
            <div id="bxc-send" class="bxc-input bxc-input-select bxc-active">
                <div class="bxc-1">
                    <input id="bxc-send-amount" type="number" autocomplete="one-time-code" />
                    <div data-value="<?php echo $default_currency ?>">
                        <img src="<?php echo BXC_URL . 'media/flags/' . strtolower(substr($default_currency, 0, 2)) . '.svg' ?>" alt="<?php echo $default_currency ?>" />
                        <span>
                            <?php echo $default_currency ?>
                        </span>
                    </div>
                </div>
                <div class="bxc-2">
                    <div class="bxc-2-1">
                        <i class="bxc-icon-search"></i>
                        <input class="bxc-search-input" type="text" name="bxc-search" placeholder="<?php bxc_e('Search...') ?>" autocomplete="one-time-code" />
                        <i class="bxc-icon-close"></i>
                    </div>
                    <div class="bxc-2-2 bxc-scrollbar">
                        <span class="bxc-title-ul bxc-title-cryptocurrencies">
                            <?php bxc_e('Cryptocurrencies') ?>
                        </span>
                        <?php bxc_html_cryptocurrencies_list() ?>
                        <span class="bxc-title-ul bxc-title-currencies">
                            <?php bxc_e('Currencies') ?>
                        </span>
                        <?php bxc_html_currencies_list(bxc_settings_get('exchange-manual-payments-active-send')) ?>
                    </div>
                </div>
            </div>
            <div id="bxc-error-pay" class="bxc-error"></div>
            <div class="bxc-title">
                <?php bxc_e(bxc_settings_get('exchange-texts-you-get', 'You get')) ?>
            </div>
            <div id="bxc-get" class="bxc-input bxc-input-select bxc-active">
                <div class="bxc-1">
                    <input id="bxc-get-amount" type="number" autocomplete="one-time-code" />
                    <div data-crypto="true" data-value="<?php echo $default_cryptocurrency ?>" data-network="<?php echo $default_cryptocurrency ?>" data-name="<?php echo bxc_crypto_name($default_cryptocurrency, true) ?>">
                        <img src="<?php echo bxc_crypto_get_image($default_cryptocurrency) ?>" alt="" />
                        <span>
                            <?php echo strtoupper($default_cryptocurrency) ?>
                        </span>
                    </div>
                </div>
                <div class="bxc-2">
                    <div class="bxc-2-1">
                        <i class="bxc-icon-search"></i>
                        <input class="bxc-search-input" type="text" name="bxc-search" placeholder="<?php bxc_e('Search...') ?>" autocomplete="one-time-code" />
                        <i class="bxc-icon-close"></i>
                    </div>
                    <div class="bxc-2-2 bxc-scrollbar">
                        <?php
                        if (bxc_settings_get('exchange-manual-payments-active-send')) {
                            echo '<span class="bxc-title-ul bxc-title-cryptocurrencies">' . bxc_('Cryptocurrencies') . '</span>';
                            bxc_html_cryptocurrencies_list(false);
                            echo '<span class="bxc-title-ul bxc-title-currencies">' . bxc_('Currencies') . '</span>';
                            bxc_html_currencies_list(true);
                        } else {
                            bxc_html_cryptocurrencies_list(false);
                        }
                        ?>
                    </div>
                </div>
            </div>
            <div class="bxc-title bxc-title-summary bxc-title-summary-start">
                <?php bxc_e(bxc_settings_get('exchange-texts-summary', 'Summary')) ?><span id="bxc-quote-update" class="bxc-flex"></span>
            </div>
            <div id="bxc-summary" class="bxc-summary">
                <div class="bxc-1"></div>
                <div class="bxc-2"></div>
            </div>
            <div id="bxc-btn-exchange-1" class="bxc-btn bxc-icon-after bxc-btn-main bxc-disabled">
                <?php bxc_e('Continue') ?>
            </div>
            <?php echo '<div class="bxc-footer">' . bxc_(bxc_settings_get('exchange-texts-footer-1', '')) . '</div>' ?>
        </div>
        <?php if ($identity_type) { ?>
            <div id="bxc-panel-identity" class="bxc-panel">
                <?php
                if ($identity_type != 'email') { ?>
                    <div class="bxc-title">
                        <?php bxc_e('First name') ?><i data-back="start" class="bxc-icon-arrow-left"></i>
                    </div>
                    <div id="bxc-first-name" class="bxc-input">
                        <input type="text" autocomplete="given-name" />
                    </div>
                    <div class="bxc-title">
                        <?php bxc_e('Last name') ?>
                    </div>
                    <div id="bxc-last-name" class="bxc-input">
                        <input type="text" autocomplete="family-name" />
                    </div>
                <?php } ?>
                <div class="bxc-title">
                    <?php echo bxc_('Email') . ($identity_type == 'email' ? ' <i data-back="start" class="bxc-icon-arrow-left"></i>' : '') ?>
                </div>
                <div id="bxc-email" class="bxc-input">
                    <input type="email" autocomplete="email" />
                </div>
                <?php
                if (bxc_settings_get('exchange-email-verification')) { ?>
                    <div id="bxc-email-verification" class="bxc-hidden">
                        <div class="bxc-title">
                            <?php echo bxc_('Verification code') ?>
                        </div>
                        <div class="bxc-text">
                            <?php echo bxc_('Enter the verification code sent to your email address and click continue.') ?>
                        </div>
                        <div id="bxc-email-verification-code" class="bxc-input">
                            <input type="number" />
                        </div>
                    </div>
                <?php } ?>
                <div id="bxc-error-identity" class="bxc-error"></div>
                <div id="bxc-btn-exchange-identity" class="bxc-btn bxc-icon-after bxc-btn-main">
                    <?php bxc_e('Continue') ?>
                </div>
                <div id="bxc-btn-exchange-identity-text" class="bxc-text">
                    <?php bxc_e('We are in the process of completing the verification. Please wait...') ?>
                </div>
                <?php
                echo '<div class="bxc-footer">' . bxc_(bxc_settings_get('exchange-texts-footer-identity', '')) . '</div>';
                if ($identity_type == 'complycube') {
                    echo '<div id="complycube-mount"></div>';
                }
                ?>
            </div>
        <?php } ?>
        <div id="bxc-panel-address" class="bxc-panel">
            <div class="bxc-title"></div>
            <div id="bxc-get-address" class="bxc-input bxc-active">
                <input type="text" autocomplete="one-time-code" />
                <img src="" alt="" />
            </div>
            <div id="bxc-error-address" class="bxc-error"></div>
            <div class="bxc-text"></div>
            <div id="bxc-btn-exchange-2" class="bxc-btn bxc-icon-after bxc-btn-main bxc-disabled">
                <?php bxc_e('Continue') ?>
            </div>
            <?php echo '<div class="bxc-footer">' . bxc_(bxc_settings_get('exchange-texts-footer-2', '')) . '</div>' ?>
        </div>
        <?php
        if (bxc_settings_get('exchange-manual-payments-active-send')) {
            $payment_methods = [['bank', 'Bank transfer'], ['card', 'Credit or debit card'], ['mobile', 'Mobile money']];
            $payment_methods_code = '<div class="bxc-title">' . bxc_('Payment method') . '<i data-back="start" class="bxc-icon-arrow-left"></i></div><div class="bxc-input"><select id="bxc-payment-method-send">';
            for ($i = 0; $i < count($payment_methods); $i++) {
                if (bxc_settings_get('exchange-manual-payments-method-' . $payment_methods[$i][0])) {
                    $payment_methods_code .= '<option value="' . $payment_methods[$i][0] . '">' . bxc_($payment_methods[$i][1]) . '</option>';
                }
            }
            $payment_methods_code .= '</select></div>';
            echo '<div id="bxc-panel-address-fiat" class="bxc-panel">' . $payment_methods_code . '<div class="bxc-title">' . bxc_('Your payment details') . '</div><div class="bxc-input"><textarea id="bxc-payment-details-send"></textarea></div><div class="bxc-text">' . nl2br(bxc_(bxc_settings_get('exchange-manual-payments-text-send', ''))) . '</div><div id="bxc-btn-exchange-2-fiat" class="bxc-btn bxc-icon-after bxc-btn-main bxc-disabled">' . bxc_('Continue') . '</div><div class="bxc-footer">' . bxc_(bxc_settings_get('exchange-texts-footer-2', '')) . '</div></div>';
        }
        ?>
        <div id="bxc-panel-payment" class="bxc-panel">
            <div class="bxc-title">
                <?php bxc_e(bxc_settings_get('exchange-texts-you-pay-with', 'You pay with')) ?><i data-back="address" class="bxc-icon-arrow-left"></i>
            </div>
            <div class="bxc-paymet-method">
                <?php
                if (bxc_settings_get('stripe-active') || bxc_settings_get('verifone-active')) {
                    echo '<div data-payment-method="' . (bxc_settings_get('stripe-active') ? 'stripe' : 'verifone') . '" class="bxc-flex bxc-active"><img src="' . BXC_URL . 'media/icon-cc.svg" alt="' . bxc_('Credit or debit card') . '" /><span>' . bxc_('Credit or debit card') . '</span></div>';
                }
                if (bxc_settings_get('paypal-active')) {
                    echo '<div data-payment-method="paypal" class="bxc-flex' . (bxc_settings_get('stripe-active') || bxc_settings_get('verifone-active') ? '' : ' bxc-active') . '"><img src="' . BXC_URL . 'media/icon-pp-2.svg" alt="PayPal" /><span>PayPal</span></div>';
                }
                if (bxc_settings_get('exchange-manual-payments-active-receive')) {
                    echo '<div data-payment-method="manual" class="bxc-flex' . (bxc_settings_get('stripe-active') || bxc_settings_get('verifone-active') || bxc_settings_get('paypal-active') ? '' : ' bxc-active') . '"><img src="' . BXC_URL . 'media/icon-manual-payment.svg" alt="' . bxc_settings_get('exchange-manual-payments-button-text', 'Manual Payment') . '" /><span>' . bxc_settings_get('exchange-manual-payments-button-text', 'Manual Payment') . '</span></div>';
                }
                ?>
                <div data-payment-method="crypto" class="bxc-flex">
                    <img src="" alt="" />
                    <div></div>
                </div>
            </div>
            <div id="bxc-payment-address-title" class="bxc-title"></div>
            <div id="bxc-payment-address" class="bxc-input bxc-active">
                <input type="text" disabled />
                <img src="" alt="" />
            </div>
            <div id="bxc-payment-address-fiat"></div>
            <div class="bxc-title bxc-title-summary bxc-title-summary-payment">
                <?php bxc_e(bxc_settings_get('exchange-texts-summary', 'Summary')) ?><span id="bxc-quote-update" class="bxc-flex"></span>
            </div>
            <div id="bxc-summary-payment" class="bxc-summary">
                <div class="bxc-1"></div>
                <div class="bxc-2"></div>
            </div>
            <div id="bxc-btn-exchange-3" class="bxc-btn bxc-icon-after bxc-btn-main bxc-disabled">
                <?php bxc_e(bxc_settings_get('exchange-texts-pay', 'Pay')) ?>
            </div>
            <?php echo '<div class="bxc-footer">' . bxc_(bxc_settings_get('exchange-texts-footer-3', '')) . '</div>' ?>
        </div>
        <div id="bxc-panel-processing" class="bxc-panel">
            <div class="bxc-loading"></div>
            <div class="bxc-title">
                <?php bxc_e('We are confirming your payment...') ?>
            </div>
            <?php echo '<div class="bxc-footer">' . bxc_(bxc_settings_get('exchange-texts-footer-4', '')) . '</div>' ?>
            <div id="bxc-cancel-verify-payment" class="bxc-link">
                <div class="bxc-icon-close"></div><?php bxc_e('Cancel') ?>
            </div>
        </div>
        <div id="bxc-panel-finish" class="bxc-panel">
            <div class="bxc-icon-check"></div>
            <div id="bxc-finish-text" class="bxc-title"></div>
            <a id="bxc-transaction-link" href="#" class="bxc-underline bxc-loading">
                <?php bxc_e('View transaction') ?>
            </a>
            <?php echo '<div class="bxc-footer">' . bxc_(bxc_settings_get('exchange-texts-footer-5', '')) . '</div>' ?>
        </div>
        <?php
        if (bxc_settings_get('exchange-manual-payments-active-receive')) {
            echo '<div id="bxc-panel-finish-manual" class="bxc-panel"><div class="bxc-icon-check"></div><div class="bxc-title">' . bxc_(bxc_settings_get('exchange-manual-payments-title-receive', '')) . '</div><div class="bxc-text">' . nl2br(bxc_(bxc_settings_get('exchange-manual-payments-text-receive', ''))) . '</div><div class="bxc-footer">' . bxc_(bxc_settings_get('exchange-texts-footer-5', '')) . '</div></div>';
        }
        ?>
    </div>
<?php } ?>